//------------------------------------------------------------------------------
// title : HexDecimal Edit Component for Delphi 3
//                          ver.0.90 / date:Nov.17,1997 / copyright:H.Gotou,1997
//------------------------------------------------------------------------------

unit EditHex;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TNumericalType = (Decimal,HexDecimal);
  TNumericalPlace = (places2, places4, places8);
  TEditHex = class(TEdit)
  private
    FAlignment: TAlignment;
    FFourDigit: Boolean;
    FMaxPlaces: TNumericalPlace;
    FNumericType: TNumericalType;
    FNumeralOnly: Boolean;
    place: integer;
    procedure SetAlignment(Value: TAlignment);
    procedure SetFourDigit(Value: Boolean);
    procedure SetMaxPlaces(Value: TNumericalPlace);
    procedure SetNumeralOnly(Value: Boolean);
    procedure SetNumericType(Value: TNumericalType);
    function GetEditText: string;
    procedure SetEditText(Value: string);
    function GetText: string;
    procedure SetText(Value: string);
    procedure WMGetDlgCode(var Msg: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure WMChar(var Msg: TWMChar); message WM_CHAR;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure KeyPress(var Key: Char); override;
    procedure Loaded; override;
    procedure FormatText; virtual;
    function InputDecimal(value: string): string; virtual;
    function InputHexDecimal(value: string): string; virtual;
    function ValueLimit(value: string): string; virtual;
    function EncodeText(value: string): string; virtual;
    function DecodeText(value: string): string; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    property EditText: string read GetEditText write SetEditText;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property FourDigit: Boolean    read FFourDigit write SetFourDigit default false;
    property MaxPlaces: TNumericalPlace   read FMaxPlaces   write SetMaxPlaces     default places8;
    property NumeralOnly: Boolean         read FNumeralOnly write SetNumeralOnly default false;
    property NumericType: TNumericalType  read FNumericType write SetNumericType default HexDecimal;
    property Text: string read GetText write SetText;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('TRSOFT', [TEditHex]);
end;

constructor TEditHex.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFourDigit   := false;
  FMaxPlaces   := places8;
  FNumeralOnly := false;
  FNumericType := HexDecimal;
  place := 8;
end;

procedure TEditHex.CreateParams(var Params: TCreateParams);
const
  Alignments: array [TAlignment] of Longint = (ES_LEFT, ES_RIGHT, ES_CENTER);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    Style := Style or Alignments[FAlignment] or ES_MULTILINE;
  end;
end;

procedure TEditHex.Loaded;
begin
  inherited Loaded;
  SendMessage(Handle, EM_SETSEL, 0, GetTextLen);
end;

procedure TEditHex.FormatText;
var
  S1, S2, S3: string;
  Len, Start, Stop, xval: Integer;
begin
  S1 := EditText;
  Len := Length(S1);
  SendMessage(Handle, EM_GETSEL, WParam(@Start), LParam(@Stop));
  Start := Len - Start;
  Stop := Len - Stop;
  if FNumericType = Decimal then
  begin
    S2 := InputDecimal(S1);
    S2 := ValueLimit(S2);
  end
  else
  begin
    S2 := '$'+InputHexDecimal(S1);
    S2 := ValueLimit(S2);
    xval := StrToInt(S2);
    S2 := '$'+IntToHex(xval,place);
  end;
  S3 := EncodeText(S2);
  Len := Length(S3);
  EditText := S3;
  Start := Len - Start;
  Stop := Len - Stop;
  SendMessage(Handle, EM_SETSEL, Start, Stop);
end;

function TEditHex.InputDecimal(value: string): string;
var
  I, Len: Integer;
  _ch: longint;
begin
  Len := Length(value);
  for I := Len downto 1 do
  begin
    _ch := Ord(value[I]);
    if not((_ch<$20)or((_ch>=$30)and(_ch<=$39))) then
    begin
      Delete(value, I, 1);
    end;
  end;
  result := value;
end;

function TEditHex.InputHexDecimal(value: string): string;
var
  I, Len: Integer;
  _ch: longint;
begin
  Len := Length(value);
  for I := Len downto 1 do
  begin
    _ch := Ord(value[I]);
    if not((_ch<$20)or((_ch>=$30)and(_ch<=$39))or((_ch>=$41)and(_ch<=$46))or((_ch>=$61)and(_ch<=$66))) then
    begin
      Delete(value, I, 1);
    end;
  end;
  result := value;
end;

procedure TEditHex.WMGetDlgCode(var Msg: TWMGetDlgCode);
begin
  inherited;
  Msg.Result := Msg.Result and not (DLGC_WANTTAB or DLGC_WANTALLKEYS);
end;

procedure TEditHex.WMChar(var Msg: TWMChar);
begin
  inherited;
  if FNumeralOnly then
  begin
    FormatText;
  end;
end;

procedure TEditHex.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if Key = Char(VK_RETURN) then
  begin
    Key := #0;
  end;
end;

procedure TEditHex.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    RecreateWnd;
  end;
end;

procedure TEditHex.SetFourDigit(Value: Boolean);
var
  S: string;
  I: integer;
begin
  if FFourDigit <> Value then
  begin
    FFourDigit := Value;
    if FNumeralOnly then
    begin
      S := DecodeText(EditText);
      I := StrToInt(S);
      if FNumericType = Decimal then
      begin
        S := IntToStr(I);
      end
      else
      begin
        S := '$'+IntToHex(I,place);
      end;
      S := ValueLimit(S);
      If FFourDigit then
      begin
        EditText := EncodeText(S);
      end
      else
      begin
        EditText := S;
      end;
    end;
  end;
end;

procedure TEditHex.SetMaxPlaces(Value: TNumericalPlace);
var
  S: string;
  I: integer;
begin
  if FMaxPlaces <> Value then
  begin
    FMaxPlaces := Value;
    if FMaxPlaces = places2 then
    begin
      place := 2;
    end;
    if FMaxPlaces = places4 then
    begin
      place := 4;
    end;
    if FMaxPlaces = places8 then
    begin
      place := 8;
    end;
    if FNumeralOnly then
    begin
      S := DecodeText(EditText);
      I := StrToInt(S);
      if FNumericType = Decimal then
      begin
        S := IntToStr(I);
      end
      else
      begin
        S := '$'+IntToHex(I,place);
      end;
      S := ValueLimit(S);
      If FFourDigit then
      begin
        EditText := EncodeText(S);
      end
      else
      begin
        EditText := S;
      end;
    end
  end;
end;

procedure TEditHex.SetNumeralOnly(Value: Boolean);
var
  S: string;
  I: integer;
begin
  if FNumeralOnly <> Value then
  begin
    FNumeralOnly := Value;
    if FNumeralOnly then
    begin
      S := DecodeText(EditText);
      S := ValueLimit(S);
      I := StrToInt(S);
      if FNumericType = Decimal then
      begin
        S := IntToStr(I);
      end
      else
      begin
        S := '$'+IntToHex(I,place);
      end;
      If FFourDigit then
      begin
        EditText := EncodeText(S);
      end
      else
      begin
        EditText := S;
      end;
    end;
  end;
end;

procedure TEditHex.SetNumericType(Value: TNumericalType);
var
  S: string;
  I: integer;
begin
  if FNumericType <> Value then
  begin
    FNumericType := Value;
    if FNumeralOnly then
    begin
      S := DecodeText(EditText);
      I := StrToInt(S);
      if FNumericType = Decimal then
      begin
        S := IntToStr(I);
      end
      else
      begin
        S := '$'+IntToHex(I,place);
      end;
      S := ValueLimit(S);
      If FFourDigit then
      begin
        EditText := EncodeText(S);
      end
      else
      begin
        EditText := S;
      end;
    end;
  end;
end;

function TEditHex.GetEditText: string;
begin
  Result := inherited Text;
end;

procedure TEditHex.SetEditText(Value: string);
begin
  inherited Text := Value;
end;

function TEditHex.GetText: string;
begin
  if FNumeralOnly then
  begin
    Result := DecodeText(EditText);
  end
  else
  begin
    Result := EditText;
  end;
end;

procedure TEditHex.SetText(Value: string);
begin
  if FNumeralOnly then
  begin
    if FNumericType=HexDecimal then
    begin
      if Value[1]<>'$' then
      begin
        value := '$'+value;
      end;
    end;
    Value := ValueLimit(Value);
    EditText := EncodeText(Value);
  end
  else
  begin
    EditText := Value;
  end;
end;

function TEditHex.ValueLimit(value: string): string;
var
  Len, I, xval: Integer;
  _Hex, _valid, detfg: boolean;
  _ch: longint;
  S: string;
begin
  _Hex := false;
  _valid := true;
  Len := Length(value);
  if Len > 0 then
  begin
    if value[1] = '$' then
    begin
      Delete(value, 1, 1);
      _Hex := true;
    end;
  end
  else
  begin
    _valid := false;
  end;
  if _Hex then
  begin
    Len := Length(value);
    for I := Len downto 1 do
    begin
      _ch := Ord(value[I]);
      if not(((_ch>=$30)and(_ch<=$39))or((_ch>=$41)and(_ch<=$46))or((_ch>=$61)and(_ch<=$66))) then
      begin
        _valid := false;
      end;
    end;
    detfg := false;
    for I := 1 to Len do
    begin
      if (value[I] = '0') and (detfg=false) then
      begin
        value[I] := '_';
      end
      else
      begin
        detfg := true;
      end;
    end;
    for I := Len downto 1 do
    begin
      if value[I] = '_' then
      begin
        Delete(value, I, 1);
      end;
    end;
    Len := Length(value);
    if (Len > place) or (Len = 0) then
    begin
      _valid := false;
    end;
    if _valid then
    begin
      xval := StrToInt('$'+value);
    end
    else
    begin
      xval := 0;
    end;
    result := '$'+IntToHex(xval,place);
  end
  else
  begin
    S := value;
    Len := Length(S);
    if Len > 0 then
    begin
      if S[1]='-' then
      begin
        Delete(S, 1, 1);
      end;
    end;
    Len := Length(S);
    for I := Len downto 1 do
    begin
      _ch := Ord(S[I]);
      if not((_ch>=$30)and(_ch<=$39)) then
      begin
        _valid := false;
      end;
    end;
    if _valid then
    begin
      if Len>10 then
      begin
        _valid := false;
      end;
      if Len=10 then
      begin
        if StrToInt(S[1])>2 then
        begin
          _valid := false;
        end
        else
        begin
          if StrToInt(S[1])=2 then
          begin
            Delete(S, 1, 1);
            xval := StrToInt(S);
            if xval > 147483647 then
            begin
              _valid := false;
            end;
          end;
        end;
      end;
    end;
    if _valid then
    begin
      xval := StrToInt(value);
    end
    else
    begin
      xval := 0;
    end;
    result := IntToStr(xval);
  end;
end;

function TEditHex.EncodeText(value: string): string;
var
  _Hex: boolean;
  Len: Integer;
begin
  _Hex := false;
  Len := Length(value);
  if Len > 0 then
  begin
    if value[1] = '$' then
    begin
      Delete(value, 1, 1);
      _Hex := true;
    end;
  end;
  Len := Length(value);
  if FFourDigit then
  begin
    while Len > 4 do
    begin
      Dec(Len, 4);
      Insert('_', value, Len + 1);
    end;
  end;
  if _Hex then
  begin
    Insert('$', value, 1);
  end;
  result := value;
end;

function TEditHex.DecodeText(value: string): string;
var
  Len, I: Integer;
begin
  Len := Length(value);
  for I := Len downto 1 do
  begin
    if value[I] = '_' then
    begin
      Delete(value, I, 1);
    end;
  end;
  result := value;
end;

end.
